import config
from Sprite import Sprite

class AreaInfo:
	def __init__(self, name, sprite, seq):
		self.name = name
		self.sprite = sprite
		self.seq = seq

class Area(Sprite):
	def __init__(self, kind, level, duration, effect_frequency = 200, fps = None):
		Sprite.__init__(self, kind.sprite, kind.seq, fps)
		
		self.duration = duration
		self.effect_freq = effect_frequency
		self.kind = kind
		self.level = level
		self.active_sum_time = 0
		self.duration_time = 0
		self.active = False

		self.duration_cbfn = None
		self.duration_cbpars = None

	def update(self, elapsed_msecs):
		Sprite.update(self, elapsed_msecs)
		self.active_sum_time += elapsed_msecs
		self.duration_time += elapsed_msecs
		if self.active_sum_time >= self.effect_freq:
			self.active_sum_time = self.active_sum_time % self.effect_freq
			self.active = True
		else:
			self.active = False

		if self.duration_cbfn is not None and self.duration_time >= self.duration:
			self.duration_cbfn(*self.duration_cbpars)

	def is_active(self):
		return self.active

	def duration_over_callback(self, cbfn, *pars):
		self.duration_cbfn = cbfn
		self.duration_cbpars = pars

class Wind(Area):
	def __init__(self, level, fps = None):
		Area.__init__(self, area_types["wind"], level, 5000 * level, 200, fps)

		self.push_force = 200

class Thunder(Area):
	def __init__(self, level, fps = None):
		Area.__init__(self, area_types["thunder"], level, 8000, 200, fps)

		self.damage = 3 + 2 * level

area_types = {
	"wind" : AreaInfo("wind", "wind", "default"),
	"thunder" : AreaInfo("thunder", "thunder", "default")
}

