import config
from Sprite import AccelSprite, MeterBar
from Missile import *
from Area import *

class EnemyInfo:
	def __init__(self, name, fly_sprite, fly_left, fly_right):
		self.name = name
		self.fly_sprite = fly_sprite
		self.fly_left = fly_left
		self.fly_right = fly_right

class Enemy(AccelSprite):
	def __init__(self, kind, level, to_left, y, max_hp, top_speed, fps = None):
		if to_left:
			AccelSprite.__init__(self, kind.fly_sprite, kind.fly_left, top_speed, fps)
			self.move(config.SCREEN_WIDTH + self.act_seq.refx - 1, y)
			self.set_speed(-self.top_speed, 0.0)
		else:
			AccelSprite.__init__(self, kind.fly_sprite, kind.fly_right, top_speed, fps)
			self.move(1 - self.base.width + self.act_seq.refx, y)
			self.set_speed(self.top_speed, 0.0)

		self.kind = kind
		self.level = level
		self.to_left = to_left
		
		self.max_hp = max_hp
		self.hp = max_hp

		self.hp_meter = MeterBar(self.base.width, 4, self.max_hp)
		self.hp_meter.set_amount(self.max_hp)

		self.phase = 0

		self.sh_info = False

	def show_info(self, sh = True):
		self.sh_info = sh
	
	def push(self, accel_x, accel_y):
		self.accel_x += accel_x
		self.accel_y += accel_y
		self.phase = 2	
		
	def update(self, elapsed_msecs):
		AccelSprite.update(self, elapsed_msecs)

		self.hp_meter.move(self.rect.left, self.rect.top - 4)
		
		# Floating in
		if self.phase == 0:
			if self.screen_position() == 0:
				if self.to_left:
					self.set_accel(50.0, 0.0)
				else:
					self.set_accel(-50.0, 0.0)
				self.phase = 1
				
		# Slowing down
		elif self.phase == 1:
			if self.act_speed <= self.top_speed / 4:
				self.set_accel(0.0, 0.0)

				self.phase = -1

		# Pushed away
		elif self.phase == 2:
			if self.act_speed >= self.top_speed:
				if self.to_left:
					if self.accel_x < 0.0:
						self.set_accel(50.0, 0.0)
						self.phase = 4
					else:
						self.set_accel(-50.0, 0.0)
						self.phase = 3
				else:
					if self.accel_x > 0.0:
						self.set_accel(-50.0, 0.0)
						self.phase = 4
					else:
						self.set_accel(50.0, 0.0)
						self.phase = 3
				
		# Pushed away recovery from reverse direction
		elif self.phase == 3:
			if self.to_left and self.speed_x <= -self.top_speed / 4:
				self.set_accel(0.0, 0.0)
				self.set_speed(-self.top_speed / 4, 0.0)
				self.phase = -1
			elif not self.to_left and self.speed_x >= self.top_speed / 4:
				self.set_accel(0.0, 0.0)
				self.set_speed(self.top_speed / 4, 0.0)
				self.phase = -1
		
		# Pushed away recovery from forward direction
		elif self.phase == 4:
			if self.to_left and self.speed_x > -self.top_speed / 4:
				self.set_accel(0.0, 0.0)
				self.set_speed(-self.top_speed / 4, 0.0)
				self.phase = -1
			elif not self.to_left and self.speed_x < self.top_speed / 4:
				self.set_accel(0.0, 0.0)
				self.set_speed(self.top_speed / 4, 0.0)
				self.phase = -1

	def draw(self, surf):
		if self.sh_info:
			self.hp_meter.draw(surf)
		AccelSprite.draw(self, surf)

	def get_hp(self):
		return self.hp

	def set_hp(self, hp):
		self.hp = hp
		if self.hp > self.max_hp:
			self.hp = self.max_hp
		elif self.hp < 0:
			self.hp = 0

		self.hp_meter.set_amount(self.hp)

	def missile_hit(self, missile):
		self.set_hp(self.get_hp() - missile.damage)

	def enter_area(self, area):
		if isinstance(area, Wind):
			if self.x < area.x:
				self.push(-area.push_force, 0.0)
			else:
				self.push(area.push_force, 0.0)

		elif isinstance(area, Thunder):
			self.set_hp(self.get_hp() - area.damage)

class Pegasus(Enemy):
	def __init__(self, level, to_left, y, fps = None):
		Enemy.__init__(self, enemy_types["pegasus"], level, to_left, y, 65 * level, 100.0, fps)

	def missile_hit(self, missile):
		if isinstance(missile, Iceball):
			damage = missile.damage / 2
		else:
			damage = missile.damage

		self.set_hp(self.get_hp() - damage)

class Drone(Enemy):
	def __init__(self, level, to_left, y, fps = None):
		Enemy.__init__(self, enemy_types["drone"], level, to_left, y, 100 * level, 80.0, fps)

	def missile_hit(self, missile):
		#if isinstance(missile, Iceball):
		#	damage = missile.damage / 2
		#else:
		#	damage = missile.damage

		self.set_hp(self.get_hp() - missile.damage)

class DarkPhoenix(Enemy):
	def __init__(self, level, to_left, y, fps = None):
		Enemy.__init__(self, enemy_types["dphoenix"], level, to_left, y, 200 * level, 80.0, fps)

	def missile_hit(self, missile):
		#if isinstance(missile, Iceball):
		#	damage = missile.damage / 2
		#else:
		#	damage = missile.damage

		self.set_hp(self.get_hp() - missile.damage)

class Dragon(Enemy):
	def __init__(self, level, to_left, y, fps = None):
		Enemy.__init__(self, enemy_types["dragon"], level, to_left, y, 250 * level, 80.0, fps)

	def missile_hit(self, missile):
		if isinstance(missile, Fireball):
			damage = -missile.damage / 2
		else:
			damage = missile.damage

		self.set_hp(self.get_hp() - damage)

enemy_types = {
	"pegasus" : EnemyInfo("pegasus", "pegasus", "fly_left", "fly_right"),
	"drone" : EnemyInfo("drone", "drone", "fly_left", "fly_right"),
	"dphoenix" : EnemyInfo("dphoenix", "dphoenix", "fly_left", "fly_right"),
	"dragon" : EnemyInfo("dragon", "dragon", "fly_left", "fly_right"),
}

enemy_classes = {
	"pegasus" : Pegasus,
	"drone" : Drone,
	"dphoenix" : DarkPhoenix,
	"dragon" : Dragon
}

